@@ -7,9 +7,8 @@ use File::DataClass::Constants qw( LOCK_BLOCKING LOCK_NONBLOCKING );
 use File::DataClass::Types     qw( Directory NonEmptySimpleStr
                                    OctalNum Path PositiveInt RegexpRef );
 use File::Spec;
-use IPC::SRLock::Functions     qw( Unspecified hash_from throw );
+use IPC::SRLock::Utils         qw( Unspecified hash_from loop_until throw );
 use Storable                   qw( nfreeze thaw );
-use Time::HiRes                qw( usleep );
 use Try::Tiny;
 use Moo;
 
@@ -19,7 +18,9 @@ extends q(IPC::SRLock::Base);
 my $_build__lockfile = sub {
    my $self = shift; my $path = $self->_lockfile_name;
 
+   # uncoverable condition false
    $path ||= $self->tempdir->catfile( $self->name.'.lck' );
+   # uncoverable branch true
    $path  =~ $self->pattern or throw 'Path [_1] cannot untaint', [ $path ];
 
    return $path;
@@ -28,7 +29,9 @@ my $_build__lockfile = sub {
 my $_build__shmfile = sub {
    my $self = shift; my $path = $self->_shmfile_name;
 
+   # uncoverable condition false
    $path ||= $self->tempdir->catfile( $self->name.'.shm' );
+   # uncoverable branch true
    $path  =~ $self->pattern or throw 'Path [_1] cannot untaint', [ $path ];
 
    return $path;
@@ -59,6 +62,17 @@ has '_shmfile_name'  => is => 'ro',   isa => NonEmptySimpleStr,
    init_arg          => 'shmfile';
 
 # Private methods
+my $_expire_lock = sub {
+   my ($self, $content, $key, $lock) = @_;
+
+   $self->log->error
+      ( $self->_timeout_error
+        ( $key, $lock->{spid}, $lock->{stime}, $lock->{timeout} ) );
+
+   delete $content->{ $key };
+   return 0;
+};
+
 my $_read_shmfile = sub {
    my ($self, $async) = @_; my ($file, $content);
 
@@ -94,6 +108,41 @@ my $_write_shmfile = sub {
    return;
 };
 
+my $_reset = sub {
+   my ($self, $key) = @_;
+   my ($lock_file, $shm_content) = $self->$_read_shmfile;
+   my $found = delete $shm_content->{ $key };
+
+   $found and $self->$_write_shmfile( $lock_file, $shm_content );
+   $lock_file->close; $found or throw 'Lock [_1] not set', [ $key ];
+   return 1;
+};
+
+my $_set = sub {
+   my ($self, $args, $now) = @_; my $key = $args->{k}; my $pid = $args->{p};
+
+   my ($lock_file, $shm_content) = $self->$_read_shmfile( $args->{async} );
+
+   if ($lock_file->have_lock) {
+      my $lock; exists $shm_content->{ $key }
+         and $lock = $shm_content->{ $key }
+         and $lock->{timeout}
+         and $now > $lock->{stime} + $lock->{timeout}
+         and $lock = $self->$_expire_lock( $shm_content, $key, $lock );
+
+      unless ($lock) {
+         $shm_content->{ $key }
+            = { spid => $pid, stime => $now, timeout => $args->{t} };
+         $self->$_write_shmfile( $lock_file, $shm_content );
+         $self->log->debug( "Lock ${key} set by ${pid}" );
+         return 1;
+      }
+   }
+
+   $lock_file->close;
+   return 0;
+};
+
 # Public methods
 sub list {
    my $self = shift; my $list = [];
@@ -111,58 +160,11 @@ sub list {
 }
 
 sub reset {
-   my $self = shift;
-   my $args = hash_from @_;
-   my $key  = $args->{k} or throw Unspecified, [ 'key' ];
-   my ($lock_file, $shm_content) = $self->$_read_shmfile; $key = "${key}";
-   my $found;
-
-   $found = exists $shm_content->{ $key } and delete $shm_content->{ $key };
-   $found and $self->$_write_shmfile( $lock_file, $shm_content );
-   $lock_file->close;
-   $found or throw 'Lock [_1] not set', [ $key ];
-   return 1;
+   my $self = shift; return $self->$_reset( $self->_get_args( @_ )->{k} );
 }
 
 sub set {
-   my $self = shift; my $args = $self->_get_args( @_ ); my $start = time;
-
-   my $key = $args->{k}; my $pid = $args->{p}; my $timeout = $args->{t};
-
-   while (1) {
-      my ($lock_file, $shm_content) = $self->$_read_shmfile( $args->{async} );
-
-      my $now = time; my $lock;
-
-      if ($lock_file->have_lock) {
-         if ($lock = $shm_content->{ $key } and $lock->{timeout}
-             and $now > $lock->{stime} + $lock->{timeout}) {
-            $self->log->error( $self->_timeout_error( $key,
-                                                      $lock->{spid   },
-                                                      $lock->{stime  },
-                                                      $lock->{timeout} ) );
-            delete $shm_content->{ $key };
-            $lock = 0;
-         }
-
-         unless ($lock) {
-            $shm_content->{ $key }
-               = { spid => $pid, stime => $now, timeout => $timeout };
-            $self->$_write_shmfile( $lock_file, $shm_content );
-            $self->log->debug( "Lock ${key} set by ${pid}" );
-            return 1;
-         }
-      }
-
-      $lock_file->close; $args->{async} and return 0;
-
-      $self->patience and $now > $start + $self->patience
-         and throw 'Lock [_1] timed out', [ $key ];
-
-      usleep( 1_000_000 * $self->nap_time );
-   }
-
-   return; # Not reached
+   my ($self, @args) = @_; return loop_until( $_set )->( $self, @args );
 }
 
 1;
@@ -171,6 +173,8 @@ __END__
 
 =pod
 
+=encoding utf-8
+
 =head1 Name
 
 IPC::SRLock::Fcntl - Set / reset locks using fcntl
@@ -282,7 +286,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2015 Peter Flanigan. All rights reserved
+Copyright (c) 2016 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
