@@ -5,8 +5,7 @@ use namespace::autoclean;
 use Cache::Memcached;
 use English                qw( -no_match_vars );
 use File::DataClass::Types qw( ArrayRef NonEmptySimpleStr Object );
-use IPC::SRLock::Functions qw( Unspecified hash_from throw );
-use Time::HiRes            qw( usleep );
+use IPC::SRLock::Utils     qw( Unspecified hash_from loop_until throw );
 use Moo;
 
 extends q(IPC::SRLock::Base);
@@ -27,108 +26,95 @@ has '_memd'    => is => 'lazy', isa => Object, reader => 'memd',
                              servers   => $_[ 0 ]->servers ) };
 
 # Private methods
-my $_sleep_or_throw = sub {
-   my ($self, $start, $now, $key) = @_;
+my $_expire_lock = sub {
+   my ($self, $data, $key, @fields) = @_;
 
-   $self->patience and $now > $start + $self->patience
-      and throw 'Lock [_1] timed out', [ $key ];
-   usleep( 1_000_000 * $self->nap_time );
-   return;
+   $self->log->error
+      ( $self->_timeout_error
+        ( $key, $fields[ 0 ], $fields[ 1 ], $fields[ 2 ] ) );
+
+   delete $data->{ $key };
+   return 0;
+};
+
+my $_list = sub {
+   my $self = shift;
+
+   if ($self->memd->add( $self->lockfile, 1, $self->patience + 30 )) {
+      my $shm_content = $self->memd->get( $self->shmfile ) // {};
+      my $list        = []; $self->memd->delete( $self->lockfile );
+
+      for my $key (sort keys %{ $shm_content }) {
+         my @fields = split m{ , }mx, $shm_content->{ $key };
+
+         push @{ $list }, { key     => $key,
+                            pid     => $fields[ 0 ],
+                            stime   => $fields[ 1 ],
+                            timeout => $fields[ 2 ] };
+      }
+
+      return $list;
+   }
+
+   return 0;
+};
+
+my $_reset = sub {
+   my ($self, $args, $now) = @_; my $key = $args->{k};
+
+   if ($self->memd->add( $self->lockfile, 1, $self->patience + 30 )) {
+      my $shm_content = $self->memd->get( $self->shmfile ) // {};
+      my $found = 0; delete $shm_content->{ $key } and $found = 1;
+
+      $found and $self->memd->set( $self->shmfile, $shm_content );
+      $self->memd->delete( $self->lockfile );
+      $found or throw 'Lock [_1] not set', [ $key ];
+      return 1;
+   }
+
+   return 0;
+};
+
+my $_set = sub {
+   my ($self, $args, $now) = @_;
+
+   my $key = $args->{k}; my $pid = $args->{p}; my $timeout = $args->{t};
+
+   if ($self->memd->add( $self->lockfile, 1, $self->patience + 30 )) {
+      my $shm_content = $self->memd->get( $self->shmfile ) // {}; my $lock;
+
+      if ($lock = $shm_content->{ $key }) {
+         my @fields = split m{ , }mx, $lock;
+
+         $fields[ 2 ] and $now > $fields[ 1 ] + $fields[ 2 ]
+            and $lock = $self->$_expire_lock( $shm_content, $key, @fields );
+      }
+
+      unless ($lock) {
+         $shm_content->{ $key } = "${pid},${now},${timeout}";
+         $self->memd->set( $self->shmfile, $shm_content );
+         $self->memd->delete( $self->lockfile );
+         $self->log->debug( "Lock ${key} set by ${pid}" );
+         return 1;
+      }
+
+      $self->memd->delete( $self->lockfile );
+   }
+
+   return 0;
 };
 
 # Public methods
 sub list {
-   my $self = shift; my $list = []; my $start = time;
-
-   while (1) {
-      if ($self->memd->add( $self->lockfile, 1, $self->patience + 30 )) {
-         my $recs = $self->memd->get( $self->shmfile ) || {};
-
-         $self->memd->delete( $self->lockfile );
-
-         for my $key (sort keys %{ $recs }) {
-            my @fields = split m{ , }mx, $recs->{ $key };
-
-            push @{ $list }, { key     => $key,
-                               pid     => $fields[ 0 ],
-                               stime   => $fields[ 1 ],
-                               timeout => $fields[ 2 ] };
-         }
-
-         return $list;
-      }
-
-      $self->$_sleep_or_throw( $start, time, $self->lockfile );
-   }
-
-   return;
+   my $self = shift; return loop_until( $_list )->( $self, { k => 'dummy' } );
 }
 
 sub reset {
-   my $self = shift; my $args = hash_from @_; my $start = time;
-
-   my $key = $args->{k} or throw Unspecified, [ 'key' ]; $key = "${key}";
-
-   while (1) {
-      if ($self->memd->add( $self->lockfile, 1, $self->patience + 30 )) {
-         my $recs = $self->memd->get( $self->shmfile ) || {}; my $found = 0;
-
-         delete $recs->{ $key } and $found = 1;
-         $found and $self->memd->set( $self->shmfile, $recs );
-         $self->memd->delete( $self->lockfile );
-         $found or throw 'Lock [_1] not set', [ $key ];
-         return 1;
-      }
-
-      $self->$_sleep_or_throw( $start, time, $self->lockfile );
-   }
-
-   return;
+   my ($self, @args) = @_; return loop_until( $_reset )->( $self, @args );
 }
 
 sub set {
-   my $self = shift; my $args = $self->_get_args( @_ ); my $start = time;
-
-   my $key = $args->{k}; my $pid = $args->{p}; my $timeout = $args->{t};
-
-   while (1) {
-      my $now = time; my ($lock_set, $rec);
-
-      if ($self->memd->add( $self->lockfile, 1, $self->patience + 30 )) {
-         my $recs = $self->memd->get( $self->shmfile ) || {};
-
-         if ($rec = $recs->{ $key }) {
-            my @fields = split m{ , }mx, $rec;
-
-            if ($fields[ 2 ] and $now > $fields[ 1 ] + $fields[ 2 ]) {
-               $recs->{ $key } = "${pid},${now},${timeout}";
-               $self->memd->set( $self->shmfile, $recs );
-
-               my $text = $self->_timeout_error
-                  ( $key, $fields[ 0 ], $fields[ 1 ], $fields[ 2 ] );
-
-               $self->log->error( $text ); $lock_set = 1;
-            }
-         }
-         else {
-            $recs->{ $key } = "${pid},${now},${timeout}";
-            $self->memd->set( $self->shmfile, $recs );
-            $lock_set = 1;
-         }
-
-         $self->memd->delete( $self->lockfile );
-
-         if ($lock_set) {
-            $self->log->debug( "Lock ${key} set by ${pid}" );
-            return 1;
-         }
-         elsif ($args->{async}) { return 0 }
-      }
-
-      $self->$_sleep_or_throw( $start, $now, $self->lockfile );
-   }
-
-   return;
+   my ($self, @args) = @_; return loop_until( $_set )->( $self, @args );
 }
 
 1;
@@ -137,6 +123,8 @@ __END__
 
 =pod
 
+=encoding utf-8
+
 =head1 Name
 
 IPC::SRLock::Memcached - Set / reset locks using libmemcache
@@ -187,10 +175,6 @@ Delete a lock from the lock table
 
 Set a lock in the lock table
 
-=head2 _sleep_or_throw
-
-Sleep for a bit or throw a timeout exception
-
 =head1 Diagnostics
 
 None
@@ -227,7 +211,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2015 Peter Flanigan. All rights reserved
+Copyright (c) 2016 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
